// Package gu provides Generic Utilities for the Go programming language.
// These utilities are low in complexity, but I use them frequently in multiple projects.
// This package depends on type parameter support, available in Go 1.18 and later.
package gu
